<?php

namespace Appvise\AppStoreNotifications;

use Appvise\AppStoreNotifications\ProxyHelperFacade;
use Illuminate\Http\Request;
use Appvise\AppStoreNotifications\Model\NotificationType;
use Appvise\AppStoreNotifications\Model\AppleNotification;
use Appvise\AppStoreNotifications\Exceptions\WebhookFailed;
use Appvise\AppStoreNotifications\Model\NotificationPayload;
use Appvise\AppStoreNotifications\ProxyHelperFacade as AppStoreNotificationsProxyHelperFacade;

class WebhooksController
{
    public function __invoke(Request $request)
    {
        $proxy_host = config('appstore-server-notifications.proxy_host');
        if (!empty($proxy_host)) {
            ProxyHelperFacade::createFromRequest($request)->toHost($proxy_host);
        }

        $jobConfigKey = NotificationType::{$request->input('notification_type')}();

        AppleNotification::storeNotification($jobConfigKey, $request->input());

        //$this->determineValidRequest($request->input('password'));

        $payload = NotificationPayload::createFromRequest($request);

        $jobClass = config("appstore-server-notifications.jobs.{$jobConfigKey}", null);

        if (is_null($jobClass)) {
            throw WebhookFailed::jobClassDoesNotExist($jobConfigKey);
        }

        $job = new $jobClass($payload);
        dispatch($job);

        return response()->json();
    }

    private function determineValidRequest(string $password): bool
    {
        if ($password !== config('appstore-server-notifications.shared_secret')) {
            throw WebhookFailed::nonValidRequest();
        }

        return true;
    }
}
